# -*- coding: utf-8 -*-
import os
import tempfile
import time
import socket
import httplib
import unittest
import smtplib
import json
import urllib
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email.mime.text import MIMEText
from email.utils import formatdate, formataddr
from email.header import Header
from email import Encoders

import lathermail
import lathermail.db


class InvalidStatus(Exception):
    def __init__(self, response):
        super(InvalidStatus, self).__init__("Invalid status {}.\n{}".format(response.status_code, response.data))
        self.response = response
        self.code = response.status_code


class SendEmailError(Exception):
    """ Exception, raised in case send is failed.
    """


class BaseTestCase(unittest.TestCase):

    inbox = "inbox"
    password = "password"
    port = 2525
    server = None
    db_name = "lathermail_test_db"
    prefix = "/api/0"
    _db_fd = None
    _db_file = None

    @classmethod
    def setUpClass(cls):
        conf = lathermail.app.config

        if os.getenv("LATHERMAIL_TEST_DB_TYPE", "sqlite") == "mongo":
            conf["DB_URI"] = conf["SQLALCHEMY_DATABASE_URI"] = "mongodb://localhost/%s" % cls.db_name
        else:
            cls._db_fd, cls._db_file = tempfile.mkstemp()
            conf["DB_URI"] = conf["SQLALCHEMY_DATABASE_URI"] = "sqlite:///%s" % cls._db_file

        lathermail.init_app()
        cls.c = lathermail.app.test_client()
        super(BaseTestCase, cls).setUpClass()
        cls.server = SmtpServerRunner(cls.db_name)
        cls.server.start(cls.port)

    @classmethod
    def tearDownClass(cls):
        super(BaseTestCase, cls).tearDownClass()
        cls.server.stop()
        if cls._db_fd is not None:
            os.close(cls._db_fd)
            os.unlink(cls._db_file)

    def setUp(self):
        lathermail.db.switch_db(self.db_name)
        super(BaseTestCase, self).setUp()

    def tearDown(self):
        from lathermail.db import drop_database

        with lathermail.app.app_context():
            drop_database(self.db_name)

    def request(self, method, url, params=None, raise_errors=True, parse_json=True, **kwargs):
        method = method.lower()
        new_kwargs = {"headers": {"X-Mail-Inbox": self.inbox, "X-Mail-Password": self.password}}
        new_kwargs.update(kwargs)
        func = getattr(self.c, method.lower())
        if params:
            params = _prepare_params(params)
            if method in ("get", "delete"):
                new_kwargs["query_string"] = urllib.urlencode(params)
            else:
                new_kwargs["data"] = params

        rv = func(self.prefix + url, **new_kwargs)
        if parse_json:
            try:
                rv.json = json.loads(rv.data)
            except ValueError as e:
                if rv.status_code != httplib.NO_CONTENT:
                    print "JSON decode error: {}, data:\n{}".format(e, rv.data)
                rv.json = None
        if raise_errors and rv.status_code >= 400:
            raise InvalidStatus(rv)
        return rv

    def get(self, url, params=None, **kwargs):
        return self.request("get", url, params, **kwargs)

    def delete(self, url, params=None, **kwargs):
        return self.request("delete", url, params, **kwargs)

    def send(self, user=None, password=None, subject="test", body="Hello"):
        smtp_send_email("test1@example.com", subject, "me@example.com", body,
                        user=user or self.inbox, password=password or self.password, port=self.port)


def _prepare_params(params):
    def convert(v):
        if isinstance(v, unicode):
            return v.encode("utf-8")
        if isinstance(v, str):
            return v
        return str(v)
    return {convert(k): convert(v) for k, v in params.iteritems()}


def prepare_send_to_field(name_email_pairs):
    return u", ".join([formataddr((str(Header(name, "utf-8")), email))
                       for name, email in name_email_pairs]).encode("utf-8")


def content_disposition(filename):
    return 'attachment; filename="{0}"'.format(Header(filename, "utf-8"))


def smtp_send_email(email, subject, from_addr, body, server_host="127.0.0.1", user=None, password=None,
                     emails=None, attachments=None, port=0):
    msg = MIMEMultipart()
    msg['To'] = email
    msg['Subject'] = subject
    msg['From'] = from_addr
    msg['Date'] = formatdate(localtime=True)
    msg.attach(MIMEText(body, _charset="utf8"))
    for name, data in attachments or []:
        part = MIMEBase('application', "octet-stream")
        part.set_payload(data)
        Encoders.encode_base64(part)
        part.add_header('Content-Disposition', content_disposition(name))
        msg.attach(part)
    try:
        s = smtplib.SMTP(server_host, port)
        if user and password:
            s.login(user, password)
        if emails is None:
            emails = [email]
        s.sendmail(from_addr, emails, msg.as_string())
        s.quit()
        #print(u"Sent email to [%s] from [%s] with subject [%s]", email, from_addr, subject)
    except (smtplib.SMTPConnectError, smtplib.SMTPException, IOError), e:
        print(u"Sending email error to [%s] from [%s] with subject [%s]:\n%s", email, from_addr, subject, e)
        raise SendEmailError(e)


class SmtpServerRunner(object):

    def __init__(self, db_name):
        self._process = None
        self.db_name = db_name

    def start(self, port=2025):
        from lathermail.db import message_handler
        from lathermail.smtp import serve_smtp
        from multiprocessing import Process

        def wrapper(**kwargs):
            lathermail.db.switch_db(self.db_name)
            serve_smtp(**kwargs)

        p = Process(target=wrapper, kwargs=dict(handler=message_handler, port=port))
        p.daemon = True
        p.start()
        self._process = p
        self.wait_start(port)

    def stop(self):
        self._process.terminate()

    def wait_start(self, port):
        timeout = 0.1
        host_port = ("127.0.0.1", port)
        for i in range(10):
            try:
                sock = socket.create_connection(host_port, timeout=timeout)
            except Exception:
                time.sleep(timeout)
                continue
            else:
                sock.close()
                return
        raise Exception("Can't connect to %s" % str(host_port))



# -*- coding: utf-8 -*-
import dateutil.parser
from flask.ext.restful import reqparse, types


def iso_date(value):
    return dateutil.parser.parse(value)


parser = reqparse.RequestParser()
parser.add_argument('X-Mail-Inbox', type=unicode, dest="inbox", location="headers")
parser.add_argument('X-Mail-Password', type=unicode, dest="password", location="headers", required=True)
parser.add_argument("sender.name", type=unicode)
parser.add_argument("sender.address", type=unicode)
parser.add_argument("recipients.name", type=unicode)
parser.add_argument("recipients.address", type=unicode)
parser.add_argument("subject", type=unicode)
parser.add_argument("subject_contains", type=unicode)
parser.add_argument("created_at_lt", type=iso_date)
parser.add_argument("created_at_gt", type=iso_date)
parser.add_argument("read", type=types.boolean)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from tests.utils import prepare_send_to_field, smtp_send_email


def main():
    inbox = "inbox"
    password = "password"
    port = 2525
    to = [("Rcpt1", "rcpt1@example.com"), ("Rcpt2", "rcpt2@example.com"), ("", "rcpt3@example.com")]
    emails = [t[1] for t in to]
    to = prepare_send_to_field(to)
    n = 3
    body_fmt = u"you you привет {}"
    subject_fmt = u"Test subject хэллоу {}"
    file_content = "file content"
    for i in range(n):
        smtp_send_email(
            to, subject_fmt.format(i), u"Me <asdf@exmapl.com>", body_fmt.format(i),
            user=inbox, password=password, port=port, emails=emails,
            attachments=[(u"tасдest.txt", file_content)]
        )


main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import email
import smtpd
import asyncore
import logging
import socket
import base64

log = logging.getLogger(__name__)


class SMTPChannelWithAuth(smtpd.SMTPChannel):

    def __init__(self, server, conn, addr):
        smtpd.SMTPChannel.__init__(self, server, conn, addr)
        self.user = None
        self.password = None
        self.fqdn = socket.getfqdn()

    def smtp_EHLO(self, arg):
        self.push("250-%s\r\n"
                  "250 AUTH PLAIN" % self.fqdn)

    def smtp_AUTH(self, arg):
        try:
            user, password = self.decode_plain_auth(arg)
        except ValueError:
            self.push("535 5.7.8  Authentication credentials invalid")
        else:
            self.push("235 2.7.0  Authentication Succeeded")
            self.user, self.password = user, password

    def smtp_MAIL(self, arg):
        if not (self.user and self.password):
            self.push("530 5.7.0  Authentication required")
            return
        smtpd.SMTPChannel.smtp_MAIL(self, arg)

    @staticmethod
    def decode_plain_auth(arg):
        user_password = arg.split()[-1]
        user_password = base64.b64decode(user_password)
        return user_password.split("\0", 3)[1:]


class InboxServer(smtpd.SMTPServer, object):

    def __init__(self, localaddr, handler):
        super(InboxServer, self).__init__(localaddr, None)
        self._handler = handler
        self._channel = None

    def handle_accept(self):
        pair = self.accept()
        if pair is not None:
            conn, addr = pair
            self._channel = SMTPChannelWithAuth(self, conn, addr)

    def process_message(self, peer, mailfrom, rcpttos, data):
        log.info("Storing message to inbox: '{0}' (from: {1}, to: {2})".format(self._channel.user, mailfrom, rcpttos))
        message = email.message_from_string(data)
        return self._handler(to=rcpttos, sender=mailfrom, message=message, body=data,
                             user=self._channel.user, password=self._channel.password)


def serve_smtp(host="127.0.0.1", port=10252, handler=None):
    if handler is None:

        def handler(to, sender, *args):
            print to, sender

    InboxServer((host, port), handler)
    log.info("Running SMTP server on %s:%s", host, port)
    asyncore.loop()



# -*- coding: utf-8 -*-
from flask import Blueprint, Response, request
from flask.ext import restful
from flask.ext.restful import Resource

from .db import find_messages, remove_messages, get_inboxes
from .validators import parser
from .representations import output_json, content_disposition


api_bp = Blueprint("api", __name__)
api = restful.Api(app=api_bp, prefix="/api/0")
api.representations.update({"application/json": output_json})


class MessageList(Resource):
    def get(self):
        args = parser.parse_args()
        messages = list(find_messages(args.password, args.inbox, args))
        return {'message_list': messages, 'message_count': len(messages)}

    def delete(self):
        args = parser.parse_args()
        remove_messages(args.password, args.inbox, args)
        return '', 204


class Message(Resource):
    def get(self, message_id):
        args = parser.parse_args()
        args["_id"] = message_id
        messages = list(find_messages(args.password, args.inbox, args, limit=1))
        if not messages:
            return {"error": "Message not found"}, 404
        return {"message_info": messages[0]}

    def delete(self, message_id):
        args = parser.parse_args()
        args["_id"] = message_id
        if remove_messages(args.password, args.inbox, args):
            return '', 204
        return {"error": "Message not found"}, 404


class Attachment(Resource):
    def get(self, message_id, attachment_index):
        args = {"_id": message_id}
        messages = list(find_messages(None, fields=args, limit=1, include_attachment_bodies=True))
        if not messages:
            return {"error": "Message not found"}, 404

        try:
            part = messages[0]["parts"][attachment_index]
        except IndexError:
            return {"error": "Attachment not found"}, 404
        return Response(
            part["body"], mimetype=part["type"],
            headers={"Content-Disposition": content_disposition(part["filename"],
                                                                request.environ.get('HTTP_USER_AGENT'))}
        )


class InboxList(Resource):
    def get(self):
        args = parser.parse_args()
        inboxes = get_inboxes(args.password)
        return {'inbox_list': inboxes, 'inbox_count': len(inboxes)}


api.add_resource(MessageList, '/messages/')
api.add_resource(Message, '/messages/<ObjectId:message_id>')
api.add_resource(Attachment, '/messages/<ObjectId:message_id>/attachments/<int:attachment_index>')
api.add_resource(InboxList, '/inboxes/')



# -*- coding: utf-8 -*-
from tests.utils import BaseTestCase, smtp_send_email, prepare_send_to_field
from lathermail.utils import utcnow


class ApiTestCase(BaseTestCase):

    def test_send_and_search(self):
        to_tuple = [("Rcpt1", "rcpt1@example.com"), ("Rcpt2", "rcpt2@example.com"), ("", "rcpt3@example.com")]
        emails = [t[1] for t in to_tuple]
        to = prepare_send_to_field(to_tuple)
        n = 3
        body_fmt = u"you you привет {}"
        subject_fmt = u"Test subject хэллоу {}"
        file_content = "file content"
        sender_name = "Me"
        sender_addr = "asdf@exmapl.com"

        for i in range(n):
            smtp_send_email(
                to, subject_fmt.format(i), u"%s <%s>" % (sender_name, sender_addr), body_fmt.format(i),
                user=self.inbox, password=self.password, port=self.port, emails=emails,
                attachments=[(u"tасдest.txt", file_content)]
            )
        res = self.get("/messages/").json
        self.assertEquals(res["message_count"], n)

        msg = res["message_list"][0]
        self.assertEquals(len(msg["parts"]), 2)
        self.assertEquals(msg["parts"][0]["body"], body_fmt.format(n - 1))
        self.assertEquals(msg["parts"][0]["is_attachment"], False)
        self.assertEquals(msg["parts"][1]["is_attachment"], True)
        self.assertIsNone(msg["parts"][1]["body"])
        self.assertEquals(len(msg["recipients"]), len(to_tuple))
        self.assertEquals([(rcpt["name"], rcpt["address"]) for rcpt in msg["recipients"]], to_tuple)
        self.assertEquals(msg["sender"]["name"], sender_name)
        self.assertEquals(msg["sender"]["address"], sender_addr)

        def msg_count(params=None):
            return self.get("/messages/", params=params).json["message_count"]

        self.assertEquals(msg_count({"subject": subject_fmt.format(0)}), 1)
        self.assertEquals(msg_count({"subject_contains": "Test"}), n)
        self.assertEquals(msg_count({"subject_contains": "no such message"}), 0)

        before_send = utcnow()
        smtp_send_email("test1@example.com", "test", "me@example.com", "Hello",
                        user=self.inbox, password=self.password, port=self.port)

        self.assertEquals(msg_count({"recipients.address": emails[0]}), n)
        self.assertEquals(msg_count({"recipients.name": "Rcpt1"}), n)

        self.assertEquals(msg_count({"recipients.address": "no_such_email@example.com"}), 0)
        now = utcnow()
        self.assertEquals(msg_count({"created_at_lt": before_send}), n)
        self.assertEquals(msg_count({"created_at_gt": before_send}), 1)
        self.assertEquals(msg_count({"created_at_lt": now}), n + 1)
        self.assertEquals(msg_count({"created_at_gt": now}), 0)

    def test_different_boxes_and_deletion(self):
        password1 = "pass1"
        password2 = "pass2"
        user = "inbox"
        n = 5

        def message_count(user, password):
            return self.get("/messages/", headers=auth(user, password)).json["message_count"]

        for i in range(n):
            self.send(user, password1)
            self.send(user, password2)

        self.assertEquals(message_count(user, password1), n)
        self.assertEquals(message_count(user, password2), n)

        one_message = self.get("/messages/", headers=auth(user, password1)).json["message_list"][0]
        self.delete("/messages/{}".format(one_message["_id"]), headers=auth(user, password1))
        self.assertEquals(
            self.delete("/messages/{}".format(one_message["_id"]),
                        headers=auth(user, password1), raise_errors=False).status_code,
            404
        )
        self.assertEquals(message_count(user, password1), n - 1)
        self.delete("/messages/", headers=auth(user, password1))
        self.assertEquals(message_count(user, password1), 0)

        n_new = 2
        new_subject = "new subject"
        for i in range(n_new):
            self.send(user, password2, subject=new_subject)

        self.assertEquals(message_count(user, password2), n + n_new)
        self.delete("/messages/", headers=auth(user, password2), params={"subject": new_subject})
        self.assertEquals(message_count(user, password2), n)

    def test_read_flag(self):
        n_read = 5
        n_unread = 3
        subject_read = "read emails"
        subject_unread = "unread emails"

        for i in range(n_read):
            self.send(subject=subject_read)
        for i in range(n_unread):
            self.send(subject=subject_unread)

        self.assertEquals(self.get("/messages/", {"subject": subject_read}).json["message_count"], n_read)
        self.assertEquals(self.get("/messages/", {"read": False}).json["message_count"], n_unread)
        self.assertEquals(self.get("/messages/", {"read": False}).json["message_count"], 0)
        self.assertEquals(self.get("/messages/").json["message_count"], n_unread + n_read)

    def test_get_inboxes(self):
        inboxes = ["first", "second", "third"]
        for inbox in inboxes:
            self.send(inbox)
        self.send("another_inbox", "another_password")
        retreived = self.get("/inboxes/", headers=auth(None, self.password)).json["inbox_list"]
        self.assertEquals(sorted(retreived), sorted(inboxes))
        self.assertEquals(self.get("/inboxes/", headers=auth(None, "unknown")).json["inbox_count"], 0)

    def test_binary_attach(self):
        binary_data = "%PDF\x93"
        smtp_send_email(
            "test@example.com", "Binary test", u"Test <asdf@exmapl.com>", u"Text body да",
            user=self.inbox, password=self.password, port=self.port,
            attachments=[("filename.pd", binary_data)]
        )
        msg = self.get("/messages/").json["message_list"][0]
        self.assertEquals(self.get("/messages/{}/attachments/{}".format(msg["_id"], 1),
                                   parse_json=False).data, binary_data)


def auth(user, password):
    return {"X-Mail-Inbox": user, "X-Mail-Password": password}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

from . import app

db_uri = app.config.get("DB_URI")
if not db_uri:
    app.config["DB_URI"] = db_uri = "sqlite:///" + os.path.expanduser("~/.lathermail.db")

if db_uri.startswith("mongodb:/"):
    app.config["MONGO_URI"] = db_uri
    from .storage.mongo import *  # noqa
else:
    app.config['SQLALCHEMY_DATABASE_URI'] = db_uri
    from .storage.alchemy import *  # noqa



# -*- coding: utf-8 -*-
import logging
from flask import Flask


app = Flask(__name__)

app.config.from_object("lathermail.default_settings")
app.config.from_envvar("LATHERMAIL_SETTINGS", silent=True)
app.config.from_pyfile("lathermail.conf", silent=True)

logging.basicConfig(level=logging.INFO, format="%(levelname)s:%(name)-16s:%(asctime)-s:%(message)s")

from . import db
from .db import init_app_for_db
from .api import api_bp
from .web import static_bp


def init_app():
    init_app_for_db(app)
    app.register_blueprint(api_bp)
    app.register_blueprint(static_bp)



import re
import json
import datetime
from urllib import quote

from flask import make_response
from bson import Timestamp, ObjectId


def output_json(data, code, headers=None):
    resp = make_response(json.dumps(data, indent=4, cls=MongoEncoder), code)
    resp.headers.extend(headers or {})
    return resp


class MongoEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime.date):
            return obj.isoformat()
        if isinstance(obj, datetime.datetime):
            return obj.isoformat()
        if isinstance(obj, Timestamp):
            return "Timestamp({}, {})".format(obj.as_datetime().isoformat(), obj.inc)
        elif isinstance(obj, ObjectId):
            return str(obj)
        elif hasattr(obj, "to_json"):
            return obj.to_json()
        else:
            return super(MongoEncoder, self).default(obj)


def content_disposition(filename, user_agent=None):
    filename = filename.encode("utf-8")
    older_msie_pattern = r"^.*MSIE ([0-8]{1,}[\.0-9]{0,}).*$"
    safari_pattern = r"^.*AppleWebKit.*$"
    user_agent = user_agent or ""

    if re.match(older_msie_pattern, user_agent, re.IGNORECASE):
        return "attachment;filename={0}".format(quote(filename))
    elif re.match(safari_pattern, user_agent, re.IGNORECASE):
        return "attachment;filename={0}".format(filename)
    return "attachment;filename*=utf-8''{0}".format(quote(filename))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import email
from email.header import decode_header
from email.utils import getaddresses


def convert_addresses(raw_header):
    result = []
    name_addr_pairs = getaddresses([raw_header])
    for name, addr in name_addr_pairs:

        result.append({"name": _header_to_unicode(name), "address": addr})
    return result


def convert_message_to_dict(to, sender, message, body, user, password):
    result = {
        "inbox": user,
        "password": password,
        "message_raw": body,
        "sender_raw": message["From"],
        "recipients_raw": message["To"],
        # for easy searching
        "sender": convert_addresses(message["From"])[0],
        "recipients": convert_addresses(message["To"]),
        "subject": _header_to_unicode(message["Subject"]),
        "read": False,
    }
    return result


def expand_message_fields(message_info, include_attachment_bodies=False):
    message = email.message_from_string(message_info["message_raw"])
    message_info["parts"] = list(_iter_parts(message, include_attachment_bodies))
    return message_info


def _iter_parts(message, include_attachment_bodies):
    parts = [message] if not message.is_multipart() else message.get_payload()

    for i, part in enumerate(parts):
        filename = part.get_filename()
        is_attachment = filename is not None
        if not include_attachment_bodies and is_attachment:
            body = None
        else:
            body = part.get_payload(decode=True)
        charset = part.get_content_charset()
        if charset:
            try:
                body = body.decode(charset)
            except Exception:
                pass

        yield {
            "index": i,
            "type": part.get_content_type(),
            "is_attachment": is_attachment,
            "filename": _header_to_unicode(filename) if filename else None,
            "charset": charset,
            "body": body,
            "size": len(body) if body else 0
        }


def _header_to_unicode(header):
    data, encoding = decode_header(header)[0]
    if encoding:
        data = data.decode(encoding)
    return data



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
import logging

from flask.ext.pymongo import PyMongo, DESCENDING

from . import ALLOWED_QUERY_FIELDS
from .. import app
from ..mail import convert_message_to_dict, expand_message_fields
from ..utils import utcnow


log = logging.getLogger(__name__)
mongo = PyMongo()


def init_app_for_db(application):
    mongo.init_app(application)


def switch_db(name):
    """ Hack to switch Flask-Pymongo db
    :param name: db name
    """
    with app.app_context():
        app.extensions['pymongo'][mongo.config_prefix] = mongo.cx, mongo.cx[name]


def message_handler(*args, **kwargs):
    msg = convert_message_to_dict(*args, **kwargs)
    msg["created_at"] = utcnow()
    with app.app_context():
        mongo.db.messages.insert(msg)


def find_messages(password, inbox=None, fields=None, limit=0, include_attachment_bodies=False):
    messages = list(_iter_messages(password, inbox, fields, limit, include_attachment_bodies))
    if messages:
        ids = [m["_id"] for m in messages]
        mongo.db.messages.update({"_id": {"$in": ids}, "read": False},
                                 {"$set": {"read": True}}, multi=True)
    return messages


def _iter_messages(password, inbox=None, fields=None, limit=0, include_attachment_bodies=False):
    query = _prepare_query(password, inbox, fields)
    for message in mongo.db.messages.find(query).sort("created_at", DESCENDING).limit(limit):
        yield expand_message_fields(message, include_attachment_bodies)


def remove_messages(password, inbox=None, fields=None):
    query = _prepare_query(password, inbox, fields)
    return mongo.db.messages.remove(query)["n"]


def get_inboxes(password):
    return mongo.db.messages.find({"password": password}).distinct("inbox")


def _prepare_query(password, inbox=None, fields=None):
    query = {}
    if password is not None:
        query["password"] = password
    if inbox is not None:
        query["inbox"] = inbox

    if fields:
        for field, value in fields.items():
            if field in ALLOWED_QUERY_FIELDS and value is not None:
                query[field] = value

        if fields.get("created_at_gt") is not None:
            query["created_at"] = {"$gt": fields["created_at_gt"]}
        if fields.get("created_at_lt") is not None:
            query["created_at"] = {"$lt": fields["created_at_lt"]}
        if fields.get("subject_contains"):
            query["subject"] = {"$regex": re.escape(fields["subject_contains"])}

    return query


@app.before_first_request
def _ensure_index():
    log.info("Ensuring DB has indexes")
    for field in list(ALLOWED_QUERY_FIELDS) + ["created_at"]:
        mongo.db.messages.ensure_index(field)


def drop_database(name):
    mongo.cx.drop_database(name)



import os

SMTP_HOST = "127.0.0.1"
SMTP_PORT = 2525
API_HOST = "127.0.0.1"
API_PORT = 5000
SQLITE_FAST_SAVE = True
SQLALCHEMY_TRACK_MODIFICATIONS = False

# This hack for tests sits here, because tests launch SMTP server subprocess
if os.getenv("LATHERMAIL_TEST_DB_TYPE", "sqlite") == "mongo":
    DB_URI = "mongodb://localhost/lathermail_test_db"



import os
from setuptools import setup, find_packages

install_requires = [
    "Flask==0.10.1",
    "Flask-RESTful==0.2.12",
    "Flask-PyMongo==0.3.0",
    "pymongo==2.7.1",
    "python-dateutil==2.2",
    "SQLAlchemy==1.0.9",
    "Flask-SQLAlchemy==2.1",
]


def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name="lathermail",
    url="https://github.com/reclosedev/lathermail/",
    version="0.1.5",
    author="Roman Haritonov",
    description="SMTP Server with API for email testing inspired by mailtrap and maildump",
    author_email="reclosedev@gmail.com",
    license="MIT",
    packages=find_packages("."),
    include_package_data=True,
    install_requires=install_requires,
    entry_points={
        'console_scripts':
            [
                'lathermail = lathermail.run_all:main',
            ]
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Web Environment",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2.7",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Server",
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import uuid
import logging

from sqlalchemy import event
from werkzeug.routing import UnicodeConverter
from flask.ext.sqlalchemy import SQLAlchemy

from . import ALLOWED_QUERY_FIELDS
from .. import app
from ..mail import convert_message_to_dict, expand_message_fields
from ..utils import utcnow, as_utc


log = logging.getLogger(__name__)
db = SQLAlchemy(app)


class Message(db.Model):
    __tablename__ = 'messages'
    _id = db.Column(db.String, primary_key=True)
    inbox = db.Column(db.String, index=True)
    password = db.Column(db.String, index=True)
    message_raw = db.Column(db.String)
    sender_raw = db.Column(db.String)
    recipients_raw = db.Column(db.String)
    subject = db.Column(db.String, index=True)
    sender_name = db.Column(db.String, index=True)
    sender_address = db.Column(db.String, index=True)
    created_at = db.Column(db.DateTime(), index=True)
    read = db.Column(db.Boolean)


class Recipient(db.Model):
    __tablename__ = 'recipients'
    name = db.Column(db.String, primary_key=True, index=True)
    address = db.Column(db.String, primary_key=True, index=True)
    message_id = db.Column(db.String, db.ForeignKey('messages._id', ondelete="CASCADE"), primary_key=True, index=True)
    message = db.relationship("Message", backref="recipients")


_message_fields = [column.name for column in Message.__table__.columns]


def init_app_for_db(application):
    if app.config.get("DEBUG_MODE"):
        logging.getLogger("sqlalchemy.engine").setLevel(logging.INFO)
    db.init_app(application)
    application.url_map.converters['ObjectId'] = UnicodeConverter

    if application.config["DB_URI"].startswith("sqlite") and application.config.get("SQLITE_FAST_SAVE"):
        @event.listens_for(db.engine, "connect")
        def _on_engine_connect(dbapi_connection, connection_record):
            cursor = dbapi_connection.cursor()
            cursor.execute("PRAGMA synchronous = 0")
            cursor.close()


def message_handler(*args, **kwargs):
    msg = convert_message_to_dict(*args, **kwargs)
    msg["_id"] = str(uuid.uuid4())
    msg["created_at"] = utcnow()
    sender = msg.pop("sender")
    msg["sender_name"] = sender["name"]
    msg["sender_address"] = sender["address"]
    recipients = msg.pop("recipients")

    with app.app_context():
        try:
            message = Message(**msg)
            message.recipients = [Recipient(name=rcp["name"], address=rcp["address"]) for rcp in recipients]
            db.session.add(message)
            db.session.commit()
        except Exception:
            import traceback
            traceback.print_exc()  # TODO


def get_inboxes(password):
    rows = db.session.query(Message.inbox).distinct(Message.inbox).filter_by(password=password)
    return [row.inbox for row in rows]


def find_messages(password, inbox=None, fields=None, limit=0, include_attachment_bodies=False):
    messages = list(_iter_messages(password, inbox, fields, limit, include_attachment_bodies))
    if messages:
        ids = (m["_id"] for m in messages)
        Message.query.filter(Message._id.in_(ids)).update({Message.read: True}, synchronize_session=False)
        db.session.commit()
    return messages


def remove_messages(password, inbox=None, fields=None):
    message_ids_query = _prepare_sql_query(password, inbox, fields, order=False, to_select=Message._id)
    count = Message.query.filter(Message._id.in_(message_ids_query)).delete(False)
    db.session.commit()
    return count


def _iter_messages(password, inbox=None, fields=None, limit=0, include_attachment_bodies=False):
    query = _prepare_sql_query(password, inbox, fields, limit=limit)
    for message in query.all():
        message = _convert_sa_message_to_dict(message)
        yield expand_message_fields(message, include_attachment_bodies)


def _convert_sa_message_to_dict(message):
    result = dict((name, getattr(message, name)) for name in _message_fields)
    result["recipients"] = [{"name": rcpt.name, "address": rcpt.address} for rcpt in message.recipients]
    result["sender"] = {"name": result.pop("sender_name"), "address": result.pop("sender_address")}
    result["created_at"] = as_utc(result["created_at"])
    return result


def _prepare_sql_query(password, inbox=None, fields=None, limit=0, order=True, to_select=Message):
    filters = []
    recipients_filters = []
    if password is not None:
        filters.append(Message.password == password)
    if inbox is not None:
        filters.append(Message.inbox == inbox)

    if fields:
        for field, value in fields.items():
            if field in ALLOWED_QUERY_FIELDS and value is not None:
                if field.startswith("recipients."):
                    sub_field = field.rsplit(".", 1)[1]
                    recipients_filters.append(getattr(Recipient, sub_field) == value)
                else:
                    filters.append(getattr(Message, field) == value)

        if fields.get("created_at_gt") is not None:
            filters.append(Message.created_at > fields["created_at_gt"])
        if fields.get("created_at_lt") is not None:
            filters.append(Message.created_at < fields["created_at_lt"])
        if fields.get("subject_contains"):
            filters.append(Message.subject.like(u"%{0}%".format(fields["subject_contains"])))

    query = db.session.query(to_select).join(Recipient).filter(db.and_(*filters))
    if recipients_filters:
        message_by_recipient = db.session.query(Recipient.message_id).filter(db.and_(*recipients_filters))
        query = query.filter(Message._id.in_(message_by_recipient))

    if order:
        query = query.order_by(Message.created_at.desc())
    if limit:
        query = query.limit(limit)
    return query


def drop_database(name):
    db.drop_all()


@app.before_first_request
def _init():
    _create_tables()


def switch_db(name):
    _create_tables()


def _create_tables():
    log.info("Ensuring DB has tables and indexes")
    db.create_all()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import argparse
from threading import Thread

from lathermail import app, init_app
from lathermail.smtp import serve_smtp
from lathermail.db import message_handler


def main():
    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.add_argument(
        "--db-uri", default=app.config["DB_URI"],
        help="DB URI, e.g. mongodb://localhost/lathermail, sqlite:////tmp/my.db"
    )
    parser.add_argument("--api-host", default=app.config["API_HOST"], help="API Host")
    parser.add_argument("--api-port", default=app.config["API_PORT"], type=int, help="API port")
    parser.add_argument("--smtp-host", default=app.config["SMTP_HOST"], help="SMTP host")
    parser.add_argument("--smtp-port", default=app.config["SMTP_PORT"], type=int, help="SMTP port")
    parser.add_argument("--debug", action="store_true", help="Run in debug mode")
    args = parser.parse_args()
    app.config["DB_URI"] = args.db_uri
    app.config["DEBUG_MODE"] = args.debug
    init_app()

    t = Thread(target=serve_smtp, kwargs=dict(handler=message_handler,
                                              host=args.smtp_host, port=args.smtp_port))
    t.daemon = True
    t.start()

    app.run(debug=False, threaded=True, host=args.api_host, port=args.api_port)


if __name__ == "__main__":
    main()



import os

from flask import send_from_directory, Blueprint

static_bp = Blueprint('static', __name__)
_static_dir = os.path.join(os.path.dirname(__file__), 'static')


@static_bp.route('/')
def index():
    return send_from_directory(_static_dir, "index.html")


@static_bp.route('/<path:filename>')
def serve_static(filename):
    return send_from_directory(_static_dir, filename)
